/*
 * This file is part of fkmines.
 * 
 * fkmines is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * fkmines is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, Richard Molitor
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <time.h>

#include "generate.h"

static int cols, lines;

bool generate (short *field, int _cols, int _lines, int mines)
{
  int i;

  if (_cols < 1 || _lines < 1 || mines > (_cols*_lines))
    return false;

  cols = _cols;
  lines = _lines;

  for (i=0; i<cols*lines; i++)
    *(field+i) = CLOSED;

  srand48 (time(0));
  while(mines)
  {
    i = lrand48 () % (cols*lines);

    if (*(field+i) != MINE)
    {
      mines--;
      *(field+i) = MINE;
      //printf ("New mine placed on %2i (%i left)\n", i, mines);

      if (i >= cols) /* not at upper border */
      {
        if (*(field + i - cols) != MINE) /* and not a mine */
          *(field + i - cols) +=1;
        if ((i % cols) && /* not at left border */
              *(field + i - cols - 1) != MINE) /* and not a mine */
          *(field + i - cols - 1)+=1;
        if (((i+1) % cols) && /* not at right border */
              *(field + i - cols + 1) != MINE) /* and not a mine */
          *(field + i - cols + 1)+=1;
      }
      //else
        //puts ("At upper border");

      if (i + cols < lines*cols) /* not at lower border */
      {
        if ( *(field + i + cols) != MINE) /* and not a mine */
          *(field + i + cols)+=1;
        if ((i % cols) && /* not at left border */
              *(field + i + cols - 1) != MINE) /* and not a mine */
          *(field + i + cols - 1)+=1;
        if (((i+1) % cols) && /* not at right border */
              *(field + i + cols + 1) != MINE) /* and not a mine */
          *(field + i + cols + 1)+=1;
      }
      //else
        //puts ("At lower border");

      if (i % cols) /* not at left border */
      {
        if (*(field + i - 1) != MINE) /* and not a mine */
            *(field + i - 1)+=1;
      }
      //else
        //puts ("At left border");

      if ((i+1) % cols) /* not at right border */
      {
        if (*(field + i + 1) != MINE) /* and not a mine */
            *(field + i + 1)+=1;
      }
      //else
        //puts ("At right border");
    }
  }

  return true;
}

void dump (const short *field, bool open)
{
  int i;

  putchar ('+');
  for (i=0; i< cols; i++)
    printf ("--");
  printf ("+\n|");

  for (i=0; i<lines*cols; i++)
  {
    if (!open)
    {
      if (*(field+i) > 8)
        printf ("%c ", CLOSED_SYM);
      else printf ("%1i ", *(field+i));
    }
    else if (*(field+i) > 9)
      printf ("%1i ", *(field+i)-10);
    else if (*(field+i) == 9)
      printf ("%c ", MINE_SYM);
    else
      printf ("%1i ", *(field+i));

    if (i+1 == lines*cols)
      printf ("|\n+");
    else if (!((i+1) % cols))
      printf ("|\n|");
  }

  for (i=0; i< cols; i++)
    printf ("--");
  printf ("+\n\n");
}

int countwhites (const short *field)
{
  int i,c;

  for (i=0, c=0; i<cols*lines; i++)
    if (!(*(field+i) % 10))
      c++;

  return c;
}
